#ifndef CARD_H
#define CARD_H

#include <stdio.h>

#define TITLE_WIDTH 40
#define TEXT_WIDTH 40
#define TEXT_HEIGHT 11

/* Logically would be 460 for 10 * (40 + 2) + 40, but it isn't */
#define MAX_TEXT_BYTES 450

/* DOS-style carriage-return linefeed */
#define TEXT_NEWLINE "\x0d\x0a"
#define TEXT_NEWLINE_LENGTH 2

struct card
{
    struct card *prev, *next;

    /* Unsigned short because text_length is uint16 in the cardfile */
    char title[TITLE_WIDTH];
    unsigned short title_length;
    char text[MAX_TEXT_BYTES];
    unsigned short text_length;

    /* OLE object or graphic bitmap skipped */
    int object_skipped;

    /* refers to whether out-of-date from disk */
    int dirty;
};

#define DIRTY_NONE -1
#define DIRTY_ERROR -2

int less_than (struct card *a, struct card *b);
int count_cards (struct card *p);

void chop_text_linefeed (char *s, int s_length, int ch);

void foreach_card (struct card *cardfile, void (*f)(struct card *card));
int foreach_card_vp
(struct card *cardfile, int (*f)(void *p, struct card *card), void *p);

/* Returns all cards in file or NULL if error */
struct card *parse_cardfile (FILE *f);

/* Writes all cards, returns success */
int write_cardfile (FILE *f, struct card *cardfile);

/* Pass in '\n' to insert newline, returns success, sets dirty fields */
int insert_char (struct card *card, int col, int row, char ch);
int insert_title_char (struct card *card, int col, char ch);

/* Returns success, sets dirty fields */
int delete_at (struct card *card, int col, int row);
int delete_prev (struct card *card, int *col, int *row);
int delete_title_at (struct card *card, int col);
int delete_title_prev (struct card *card, int *col);

/* Whether card has been changed since being created or loaded */
int get_dirty_flag (struct card *card, int *flag);

int num_rows (struct card *c);
int max_row (struct card *c);

int row_num_cols (struct card *c, int row);

/* Not including NL */
/* Can return -1 if row is zero-width */
int row_max_col (struct card *c, int r);

/* Returns success */
int get_row (struct card *card, char *buffer, int *row_len, int row);

/* Must call sort_cardfile after setting title */
void set_title (struct card *card, char *buffer, int  title_length);
void get_title (struct card *card, char *buffer, int *title_length);

/* Pointer bubble sort (usually only one card out of order) */
/* Depends on current locale */
struct card *sort_cardfile (struct card *cardfile);

/* Returns success */
int duplicate_card (struct card *card);

/* Returns allocated cardfile or NULL */
struct card *alloc_blank_cardfile (int n);

/* Returns allocated card or NULL */
struct card *alloc_blank_card (void);

/* Returns allocated card or NULL */
struct card *prepend_blank_card (struct card *cardfile);

/* Returns next card, or previous card if no next card, or NULL */
struct card *delete_card (struct card *card);

/* Frees all cards, forward- and back-linked */
void delete_cardfile (struct card *cardfile);

#endif /* CARD_H */
