#ifndef CARD_PRIVATE_H
#define CARD_PRIVATE_H

#include "card.h"
#include "utility.h"

#include <stdio.h>

struct card_index
{
    unsigned long card_offset;
    unsigned char flag;

    unsigned char title[TITLE_WIDTH];
    unsigned long title_length;
};

enum cardfile_type { MGC, RRG, DKO };

/* Returns count of cards. Must call head before calling */
int count_cards_priv (struct card *p);

int row_from_location (struct card *c, char *p);
int col_from_location (struct card *c, char *p);
int write_mgc_header (FILE *f, struct card *cardfile);
int write_indexes (FILE *f, struct card *cardfile);
int write_one_index (FILE *p, struct card *c, int offset);
int write_one_mgc_card (void *p, struct card *c);
int bytes_one_mgc_card (void *p, struct card *c);

int insert_newline_private (struct card *c, int col, int row);
int insert_char_private (struct card *c, int col, int row, int ch);
int insert_title_char_private (struct card *c, int col, int ch);
char *locate_position (struct card *c, int col, int row);

/* Used by sort_cardfile and insert_blank_card */
struct card *head (struct card *card);
int sort_pass (struct card *c);

/* Returns success */
int determine_type (FILE *f, enum cardfile_type *crd_type);
int get_num_cards (FILE *f, enum cardfile_type crd_type,
                   unsigned short *n);
int get_card_indices (FILE *f, enum cardfile_type crd_type,
                      struct card_index *cdx, unsigned short n);

/* Returns success */
int get_card (FILE *f, enum cardfile_type crd_type,
              struct card_index *cdx, struct card *card);

int get_next_row (char **pos, char *max);

/* Should be standard, but it's not. Uses inefficient algorithm */
void *memmem (void *haystack, int haystack_length,
              void *needle, int needle_length);

#endif /* CARD_PRIVATE_H */
