/* A state assumes ownership of a FILE * and cardfile. It will close and free
   them when state_free is called. */

struct state
{
    struct card *cur_card;
    FILE *file;
    int cursor_col, cursor_row;
    int screen_cols, screen_rows;
    struct card *first_index, *last_index;
    enum { HELP, SKIPPED, LOADED, SAVED } status;
    int title_mode, title_col;
};

struct state *state_new (void);
void state_compute_indexes (struct state *s);       
void state_duplicate_card (struct state *s);
void state_inform_cards (struct state *s, struct card *cards);
void state_inform_file (struct state *s, FILE *file);
void state_insert (struct state *s, int ch);
void state_step_card (struct state *s, int n, int refresh_ok);
void state_jump_card (struct state *s, struct card *c, int refresh_ok);
void state_move_cursor (struct state *s, int dir);
void state_new_card (struct state *s);
void state_delete_card (struct state *s);
void state_refresh (struct state *s);
void state_save (struct state *s);
void state_delete_prev (struct state *s);
void state_delete_at (struct state *s);
void state_free (struct state *s);
void state_inform_width (struct state *s, int w);
void state_inform_height (struct state *s, int h);
void state_status_update (struct state *s);

/* For state_step_card */
#define NO_REFRESH 0
#define REFRESH_OK 1
